@@ -9,9 +9,9 @@
 # Copyright (C) 2000-2003 Rogério Reis & Nelma Moreira {rvr,nam}@ncc.up.pt
 #
 
-from yappy.parser import *
+#from yappy.parser import *
 import sys, string
-import parser
+from  parser import *
 ############## Demos  #####################
 
 class SimpleExp(Yappy):
@@ -21,7 +21,7 @@ class SimpleExp(Yappy):
         grammar = grules([
                 ("E -> E + T", self.Add),
                 ("E ->T", DefaultSemRule),
-                ("T -> T * F", self.Mul),
+                ("T -> T * F", self.Mul ),
                 ("T -> F", DefaultSemRule),
                 ("F -> ( E )", self.ParSemRule),
                 ("F -> id", DefaultSemRule)]
@@ -31,9 +31,10 @@ class SimpleExp(Yappy):
                ("\+",lambda x: (x,x)),
                ("\*",lambda x: (x,x)),
                ("\(|\)",lambda x: (x,x)) ]
-        Yappy.__init__(self,tokenize,grammar,table,no_table)
+        Yappy.__init__(self,tokenize,grammar,table,no_table,
+    tmpdir='/tmp')
         
-
+    
     def ParSemRule(self,list,context=None):
         return list[1]
 
@@ -54,6 +55,25 @@ class SimpleExp(Yappy):
         print "Input: %s" %st
         print "Result:", self.input(st)   
 
+class SimpleExp3(SimpleExp):
+    """ A parser  for simple arithmetic expresssion. Allows blanks
+    between tokens"""
+    def __init__(self,no_table=0, table='saexp.tab'):
+        grammar = """ 
+                E -> E + T {{ "Sum($0,$2)"}};
+                E ->T ;
+                T -> T * F" {{ self.Mul }};
+                T -> F ;
+                F -> ( E ) {{ self.ParSemRule}};
+                F -> id;
+                """
+        tokenize=[("\s+",""),
+               ("\d+",lambda x: ("id",int(x))),
+               ("\+",lambda x: (x,x)),
+               ("\*",lambda x: (x,x)),
+               ("\(|\)",lambda x: (x,x)) ]
+        Yappy.__init__(self,tokenize,grammar,table,no_table)
+        
 class SimpleExp1(Yappy):
     """ A parser  for simple arithmetic expresssions, with operators """
     def __init__(self,no_table=0, table='saexp1.tab', tabletype=LALRtable,noconflicts=1,expect=0):
@@ -95,14 +115,17 @@ class SimpleExp1(Yappy):
         return apply(list[1],[list[0],list[2]])
 
     def test(self):
-        st = "2-24*9-34*2+1"
+        st = "2-24*9"
+        st1 = "2-24*9-34*2+1"
         print "Input: %s" %st
         print "Result:", self.input(st)
 
 
 class SimpleExp2(SimpleExp1):
     """ A parser  for simple arithmetic expresssions with prec and associativity"""
-    def __init__(self,no_table=0, table='saexp2.tab', tabletype=LALRtable,noconflicts=1,expect=0):
+    def __init__(self,no_table=0, table='saexp2.tab',
+    tabletype=LALRtable,noconflicts=1,expect=0):
+        self.line = 0
         grammar = grules([
                 ("E -> E add_op T", self.Add),
                 ("E ->T", DefaultSemRule),
@@ -112,12 +135,17 @@ class SimpleExp2(SimpleExp1):
                 ("F -> id", DefaultSemRule)])
 
         tokenize=[("\d+",lambda x: ("id",int(x))),
+                  ("\n+",lambda x: (x,self.countline())),
                ("[+-]",lambda x: ("add_op",self.make_op(x)),("add_op",100,'left')),
                ("[*/]",lambda x: ("mul_op",self.make_op(x)),("mul_op",200,'left')),
                ("\(|\)",lambda x: (x,x)) ]
         Yappy.__init__(self,tokenize,grammar,table,no_table,tabletype,noconflicts,expect)
         
 
+    def countline(self):
+        self.line+=1
+        return ""
+    
     def make_op(self,op):
         """ """
         return {"+"  : operator.add,
@@ -144,21 +172,58 @@ class SimpleExp2(SimpleExp1):
 
 class SimpleExpAmb(SimpleExp2):
     """A parser  for simple arithmetic expresssions with an ambiguous grammar """
-    def __init__(self,no_table=1, table='expamb.tab',tabletype=LALRtable,noconflicts=1,expect=0):
+    def __init__(self,no_table=0, table='expamb.tab',tabletype=LALRtable,noconflicts=1,expect=0):
         grammar = grules([
                 ("E -> E add_op E", self.Add),
                 ("E -> E mul_op E", self.Mul),
                 ("E -> ( E )", self.ParSemRule),
-                
                 ("E -> id", DefaultSemRule)])
-        
-        tokenize=[("\d+",lambda x: ("id",int(x))),
-               ("[+-]",lambda x: ("add_op",self.make_op(x)),("add_op",100,'left')),
+
+        sinal = "[+-]"
+        integer ="\d" 
+        tokenize=[("(%s)+"%integer,lambda x: ("id",int(x))),
+               ("%s"%sinal,lambda x: ("add_op",self.make_op(x)),("add_op",100,'left')),
                ("[*/]",lambda x: ("mul_op",self.make_op(x)),("mul_op",200,'left')),
                ("\(|\)",lambda x: (x,x)) ]
 
         Yappy.__init__(self,tokenize,grammar,table,no_table,tabletype,noconflicts,expect)
 
+
+class SimpleExpAmb2(SimpleExp2):
+    """A parser  for simple arithmetic expresssions with an ambiguous
+    grammar, and context-dependent precedence """
+    def __init__(self,no_table=0, table='expamb.tab',tabletype=LALRtable,noconflicts=1,expect=0):
+        grammar = grules([
+                ("E -> E add_op E", self.Add),
+                ("E -> E mul_op E", self.Mul),
+                ("E -> n_op E", lambda l,c: -1*l[1]),
+                ("E -> ( E )", self.ParSemRule),
+                
+                ("E -> id", DefaultSemRule)])
+
+        plus = "[+-]"
+        integer = "\d" 
+        tokenize=[("(%s)+"%integer,lambda x: ("id",int(x))),
+               ("%s"%plus,lambda x:
+                ("add_op",self.make_op(x)),("add_op",100,'left')),
+                ("~",lambda x: ("n_op",self.make_op('-')),("n_op",300,'left')),
+               ("[*/]",lambda x: ("mul_op",self.make_op(x)),("mul_op",200,'left')),
+               ("\(|\)",lambda x: (x,x)) ]
+
+        Yappy.__init__(self,tokenize,grammar,table,no_table,tabletype,noconflicts,expect)
+    def test(self):
+        st=[
+         "~2",
+        "2-24*9",
+        "2-24*9-34*2+1",
+        "~2-24*9-34*2+1",
+         "2+3+(~5*(2+3)*2)-24*9-34*2+1"
+         ]
+        for i in st:
+            print "Input: %s" %i
+            print "Result:", self.input(i)
+
+
 class ListAVG(Yappy):
     """A parser  for transforming a list atrib=value into a python dictionary """
     def __init__(self,no_table=0, table='Listavg.tab'):
@@ -239,6 +304,7 @@ class ListAVG2(ListAVG):
 
     def test(self):
         st = "(a=5,b=6,c=7,)"
+        
         print "Input: %s" %st      
         self.input(st,context={})
         print self.context
@@ -281,9 +347,12 @@ class RegExp(Yappy):
          return "(%s*)" %list[0]
 
      def test(self):
-         st  = "(a+b)*.a.a.b*"
-         print "Input: %s" %st
-         print self.input(st)
+         st  = ["(a+b)*.a.a.b*",
+                "a+a.b+a.b.(a+a)*",
+                 "a+a.b*.(a+b)"]
+         for i in st:
+             print "Input: %s" %i
+             print self.input(i)
 
 class RegExp1(RegExp):
      def __init__(self,no_table=0, table='tableambreg1',tabletype=LALRtable,
@@ -296,7 +365,7 @@ class RegExp1(RegExp):
                                ("reg -> id",self.BaseSemRule)
                    ])
         tokenize =[
-                    ("[A-Za-z0-9]",lambda x: ("id",x),("id",400,'left')),
+                    ("[A-Za-z0-9]",lambda x: ("id",x)),
                     ("[+|]",lambda x: ("+",x),("+",100,'left')),
                     ("[*]",lambda x: (x,x),("*",300,'left')),
                     ("\(|\)",lambda x: (x,x)) ]
@@ -305,30 +374,43 @@ class RegExp1(RegExp):
      def ConcatSemRule(self,list,context=None):
          return "(%s%s)" %(list[0],list[1])
 
+
      def test(self):
          st  = ["(a+b)*aab*",
                 "(a+ab)*a*",
+                "(a+a)a+ab",
+                "(a+ab)(a+ab)(ac+a)",
                 "a+abc+ad",
-                "a+ab*",
+                "abc+b+ad",
+                "a+ab",
+                "a+b+ab+cccaaaaaa",
+                "a+ab(a+a)",
                 "ab+ab(a+a)a*",
                 "a+ab*",
-                "(a+ab*(a+b))*"]
+                "(a+ab*(a+b))*",
+                "a+ab*(a+b)",
+                "a+c+ab(a+b)",
+                "a+c+(a+b)ab",
+                "a+b*",
+                "aa+b*"
+                ]
          for i in st:
              print "Input: %s" %i
              print self.input(i)
 
+
 class RegExp2(RegExp1):
      def __init__(self,no_table=0, table='tableambreg2'):
          grammar = """
         reg -> reg + reg {{ self.OrSemRule }} |
-               reg reg {{ self.ConcatSemRule // 200 left }} |
+               reg reg {{ self.ConcatSemRule}} // 200 left|
                reg * {{ self.StarSemRule }} |
                ( reg ) {{self.ParSemRule }} |
                id {{ self.BaseSemRule }} ;
         """
-         tokenize = [("@epsilon",lambda x: ("id",x),("id",400,'left')),
-                    ("@empty_set",lambda x: ("id",x),("id",400,'left')),
-                    ("[A-Za-z0-9]",lambda x: ("id",x),("id",400,'left')),
+         tokenize = [("@epsilon",lambda x: ("id",x)),
+                    ("@empty_set",lambda x: ("id",x)),
+                    ("[A-Za-z0-9]",lambda x: ("id",x)),
                     ("[+|]",lambda x: ("+",x),("+",100,'left')),
                     ("[*]",lambda x: (x,x),("*",300,'left')),
                     ("\(|\)",lambda x: (x,x)) ]
@@ -360,12 +442,20 @@ class RegExp3(RegExp):
          print "Input: %s" %st
          print self.input(st)
         
-         
+def Sum(a,b):
+    return a+b
+
+def curry(f,*a,**kw):
+    def curried(*more_a,**more_kw):
+        return f(*(a+more_a),**dict(kw,**more_kw))
+    return curried
+
 if __name__ == '__main__':
-    d = SimpleExpAmb(1)
+    d = SimpleExpAmb()
     st = "2-24*9"
     print "Input:", st
     print "Result:", d.input(st)
-    st = "2-24K*9-34*2+1"
+    st = "2-24*9-34*2+1"
     print "Input:", st
     print "Result:", d.input(st)
+    
