@@ -10,7 +10,7 @@ A LR parser generator, based on Aho and al. 1986, C{Compilers}
 It currently builds C{SLR}, C{LR(1)} and  C{LALR(1)} parsing tables.
 
 Copyright (C) 2000-2003 Rogério Reis & Nelma Moreira {rvr,nam}@ncc.up.pt
-Version: $Id: parser.py,v 1.9 2004/12/02 10:17:50 nam Exp $
+Version: $Id: parser.py,v 1.17 2006-07-19 08:21:27 nam Exp $
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -35,9 +35,11 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 """
 
 from types import *
-import re, exceptions
+import re, exceptions, string
 import sys, string, copy, time, operator
 
+import os.path
+
 import shelve
 
 # set elements are mutable objects; we cannot use sets
@@ -45,9 +47,9 @@ import osets
 
 #Globals
 
-_DEBUG = 0
+_DEBUG=0
 
-_Version = 1.2
+_Version = 1.7
 
 NIL = ""
 
@@ -86,19 +88,24 @@ class Lexer:
         self.rules = []
         rnumber = 1
         for r in rules_list:
-            rex = r[0]
-            funct = r[1]
+            try: 
+                rex = r[0]
+                funct = r[1]
+            except IndexError:
+                raise LexicalError(rnumber)
             try: rec  = re.compile(rex)
             except TypeError:
                 raise LexicalRulesErrorRE(rex,rnumber)
             try:
                 op,prec,assoc = r[2]
                 if not self.__dict__.has_key("operators"):
-                    self.operators={}
+                    self.operators = {}
                 if not self.operators.has_key(op):
-                    self.operators[op]=(prec,assoc)
+                    self.operators[op] = (prec,assoc)
             except IndexError:
                 pass
+#                print "Error in rule %s\n", rnumber
+#                raise LexicalError(rnumber,r)
             self.rules.append((rec,funct))
             
             rnumber = rnumber + 1 
@@ -182,7 +189,9 @@ class YappyError(Exception):
 
 class LexicalError(YappyError):
     """Class for all Yappy Lexical analyser exceptions"""
-    pass
+    def __init__(self,r,rule):
+        self.message = 'Error in rule number %s'%(r)
+        print rule
 
 class LexicalRulesErrorRE(YappyError):
     """An error occured parsing the RE part of a lexical rule"""
@@ -236,10 +245,12 @@ class LRParserError(YappyError):
 
 class SemanticError(YappyError):
     """An error occured in the application of a semantic action"""
-    def __init__(self,m):
+    def __init__(self,m,n=0,r=None):
        self.message = m
+       self.nrule = n
+       self.rule = r
     def __str__(self):
-       return "%s" % (self.message)
+       return "%s in semantic rule %d: %s" % (self.message,self.nrule,self.rule)
 
 class TableError(YappyError):
     """Mismatch table version """
@@ -654,8 +665,8 @@ class LRtable:
         self.gr.makeFFN()
         self.operators = operators
         self.precedence = None
-        if self.operators:
-            self.rules_precedence()
+#        if self.operators:
+        self.rules_precedence()
         self.Log=LogLR(noconflicts,expect)
         self.make_action_goto()
 
@@ -697,19 +708,22 @@ class LRtable:
                     pass
 
     def rules_precedence(self):
-        """Rule precedence obtained as the precedende of the right
+        """Rule precedence obtained as the precedence of the right
         most terminal. """
         self.precedence={}
         for i in range(len(self.gr.rules)):
             if len(self.gr.rules[i]) == 4:
                 self.precedence[i] = self.gr.rules[i][3]
-            else:    
-                self.gr.rules[i][1].reverse()
-                for s in self.gr.rules[i][1]:
-                    if self.operators.has_key(s):
-                        self.precedence[i] = self.operators[s]
-                        break
-                self.gr.rules[i][1].reverse()  
+            else:
+                self.precedence[i] = 0
+                if self.operators:
+                    self.gr.rules[i][1].reverse()
+                    for s in self.gr.rules[i][1]:
+                        if self.operators.has_key(s):
+                            self.precedence[i] = self.operators[s]
+                            break
+                    self.gr.rules[i][1].reverse()
+
         if _DEBUG:        
             print "Precedence %s" %self.precedence
         
@@ -753,7 +767,7 @@ class LRtable:
             if self.operators and self.operators.has_key(a):
                 prec_op, assoc_op = self.operators[a]
             else:
-                prec_op, assoc_op  = (0,'DUMMY')
+                prec_op, assoc_op  = (10000,'DUMMY')
             if self.precedence.has_key(r):
                 if (self.precedence[r][0] > prec_op) or (self.precedence[r][0] == prec_op and    self.precedence[r][1] =='left'):
                     self.ACTION[(i,a)] = ('reduce',r)
@@ -1346,7 +1360,10 @@ class LRparser:
         @type noconflicts: integer
         @param expect: exact number of expected LR shift/reduce conflicts
         @type expect: integer
-        
+        @param args: extra arguments; key C{nosemrules} if 1 no
+        semantic rules are applied
+        @type args: dictionary
+
         """
 
         self.cfgr = CFGrammar(grammar)
@@ -1354,7 +1371,11 @@ class LRparser:
         self.terminals = self.cfgr.terminals
         self.nonterminals = self.cfgr.nonterminals
         self.endmark = self.cfgr.endmark
-        
+        if args.has_key('nosemrules'):
+             self.nosemrules=args['nosemrules']
+        else:
+            self.nosemrules = 0
+            
         d = shelve.open(table_shelve)
 
         if d and no_table:
@@ -1464,9 +1485,12 @@ class LRparser:
                 elif  self.ACTION[s,a][0] == 'reduce':
                     n =  self.ACTION[s,a][1]
                     if _DEBUG: print "Action: reduce %s %s\n" %(n,str(self.rules[n]))
-                    semargs = [self.stack.pop()[1] for i in range(len(self.rules[n][1]))]
-                    semargs.reverse()    
-                    reduce = Reduction(self.rules[n][2],semargs,self.context)
+                    semargs = [self.stack.pop()[1] for i in   range(len(self.rules[n][1]))]
+                    semargs.reverse()
+                    if self.nosemrules:
+                        reduce = []
+                    else:
+                        reduce = Reduction(self.rules[n][2],semargs,self.context)
                     del semargs
                     s1 = self.stack.top()[0]
                     a = self.rules[n][0]
@@ -1477,11 +1501,14 @@ class LRparser:
                 else:
                     raise LRParserError(s,a)
             except KeyError:
-                if _DEBUG: print "Error in action: %s" %self.ACTION
+                if _DEBUG: print "Error in action: %s" %self.ACTION 
                 raise LRParserError(s,a)
+            except SemanticError, m:
+                if _DEBUG: print "Semantic Rule %d %s" %(n,self.rules[n][2])
+                raise SemanticError(m,n,self.rules[n][2])
         return  self.stack.top()[1]
 
-    def parse_grammar(self,st,context):
+    def parse_grammar(self,st,context,args):
          """
          Transforms a string  into a grammar description
 
@@ -1489,7 +1516,8 @@ class LRparser:
         default symbols as below. Fisrt rule for start.
 
         I{Example}::
-             reg -> reg + reg E{lb}E{lb} self.OrSemRule // (priority,'left') E{rb}E{rb} |
+             reg -> reg + reg E{lb}E{lb} self.OrSemRule   E{rb}E{rb}
+             // priority 'left'|
                          ( reg ) E{lb}E{lb}self.ParSemRuleE{rb}E{rb} ;
         where:
        
@@ -1500,7 +1528,7 @@ class LRparser:
          -  csemsym=E{rb}E{rb} semantic rule end marker
          -  rulesep='|' separator for multiple rules for a LHS
          -  ruleend=';' end marker for one LHS rule"""
-         self.pg=Yappy_grammar()
+         self.pg=Yappy_grammar(**args)
          self.pg.input(st,context)
          return self.pg.context['rules']
     
@@ -1637,7 +1665,8 @@ def Dict(**entries):
 
 def grules(rules_list,rulesym="->",rhssep=None):
     """
-    Transforms a list of rules in a grammar description
+    Transforms a list of rules in a grammar description. If a rule has
+    no semantic rules, C{DefaultSemRule} is assumed.
     
     @param rules_list: is a list of pairs (rule,sem)
          where rule is a string  of the form:
@@ -1650,7 +1679,10 @@ def grules(rules_list,rulesym="->",rhssep=None):
     gr = []
     sep = re.compile(rulesym)
     for r in rules_list:
-        rule = r[0]
+        if type(r) is StringType:
+            rule = r
+        else:    
+            rule = r[0]
         m = sep.search(rule)
         if not m: continue
         else:
@@ -1665,10 +1697,15 @@ def grules(rules_list,rulesym="->",rhssep=None):
                     rhs = []
                 else:    
                     rhs = string.split(rest,rhssep)
-        if len(r)==3:            
+        if type(r) is StringType:
+              gr.append((lhs,rhs,DefaultSemRule))            
+        elif len(r)==3:            
             gr.append((lhs,rhs,r[1],r[2]))
-        else:
+        elif len(r)==2:
              gr.append((lhs,rhs,r[1]))
+        else:
+           raise GrammarError(r)
+
     return gr
 
 #######################################################
@@ -1680,47 +1717,61 @@ class Yappy(LRparser):
         """
     
     def __init__(self,tokenize,grammar, table='YappyTab', no_table=1,
-                 tabletype=LALRtable,noconflicts=1,expect=0):
+                 tabletype=LALRtable,noconflicts=1,expect=0,**args):
         """
         @param tokenize: same as for L{Lexer}
         @param grammar: if a string C{parse_grammar} is called
 
         @param table: and no_table, tabletype same as for L{LRparser}
-        
+
+        @param **args: dictionary where:
+        - key C{tmpdir} is the directory
+  where the parse table used by the Yappy Grammar is stored;
+    -  key  C{usrdir} is the directory where the user tables are stored
+    -  key  C{nosemrules} if 1 semantic actions are not applied
         """
         self.lex = Lexer(tokenize)
         operators = None
         if self.lex.__dict__.has_key("operators"):
             operators = self.lex.operators
         if type(grammar) is StringType:
-            grammar = self.parse_grammar(grammar,{'locals':locals()})
-        LRparser.__init__(self,grammar,table,no_table,tabletype,operators,noconflicts,expect)
+            grammar = self.parse_grammar(grammar,{'locals':locals()},args)
+        if args.has_key('usrdir') and  os.path.isdir(args['usrdir']):
+                table =  string.rstrip(args['usrdir']) + '/' + table
+        if os.path.dirname(table)=="" or os.path.exists(os.path.dirname(table)):
+            LRparser.__init__(self,grammar,table,no_table,tabletype,operators,noconflicts,expect,**args)
+        else:
+            sys.stderr.write("Directory %s do not exist\n" %table)
+            sys.exit()
         if (self.Log.noconflicts and ((self.Log.conflicts.has_key('sr') and
                                  len(self.Log.conflicts['sr'])!=
         self.Log.expect) or self.Log.conflicts.has_key('rr'))):
-            print "LR conflicts %s" %self.Log.conflicts
+            print "LR conflicts: number %s value %s" %(len(self.Log.conflicts['sr']),self.Log.conflicts)
             print """If it is Ok, set expect to the number of conflicts and build table again"""
-    def  input(self,str=None,context=None):
+
+    def  input(self,str=None,context={},lexer=0):
         """ Reads from stdin or string and retuns parsed result
 
             @param str: String to be parsed. If not given, reads from
             C{stdin}.
             @param context: some initial computational context
+            @param lexer: if 1 only lexical analisys is performed  
             
             @return: a tuple C{(parsed result,context)} or
             only the C{parsed result}
             
         """
-
         if str:
             self.tokens = self.lex.scan(str)
         else:     
             print "Input:  ",
             self.tokens = self.lex.readscan()
-        self.context = context    
+        if lexer:
+            return self.tokens
+        self.context = context
         return self.parsing(self.tokens,self.context)
         
-    def inputfile(self,FileName,context=None):
+    def inputfile(self,FileName,context={}):
         """Reads input from file """
         try: file = open(FileName,"r")
         except IOError: raise YappyError()
@@ -1736,25 +1787,49 @@ class Yappy(LRparser):
         pass
     
 ######### Semantic Grammar Rules ##############
+def expandSemRule(strargs,strfun):
+    regargs = re.compile(r'\$(\d+)')
+    matchargs = regargs.finditer(strfun)
+    for i in [(x.group(0),strargs+x.group(1)+"]") for x in matchargs]:
+        strfun = string.replace(strfun,i[0],i[1])
+    return strfun    
+        
+def Reduction(fun,sargs,context={}):
+    """Reduction function for semantic rules:
+    - C{fun} can be:
+      -- a function
+      -- or a string with positional arguments C{$n} that is expanded
+    and evaluated with C{eval}
+    
+    """
+    if callable(fun):
+        try:
+            return apply(fun,[sargs, context])
+        except:
+            raise SemanticError,'Wrong application: %s %s' %(fun,sargs)
+    elif  type(fun) is StringType:
+        a = expandSemRule("sargs[",fun)
+        l = context.get('locals',{})
+        l.update(locals())
+        return  eval(a,context.get('globals',{}),l)
+    else:
+        raise SemanticError,'Wrong type: %s' %fun
 
-def Reduction(fun,list,context=None):
-    """Reduction function for semantic rules"""
-    return apply(fun,[list,context])
-
-def DefaultSemRule(list,context=None):
+   
+def DefaultSemRule(sargs,context={}):
     """Default  semantic rule"""
-    return list[0]
+    return sargs[0]
 
-def EmptySemRule(list,context=None):
+def EmptySemRule(sargs,context={}):
     return []
 
 
    
-######Parser for grammars ##################
+######Parser f,grammars ##################
 class Yappy_grammar(Yappy):
     """ A parser for grammar rules. See C{test()} for an example. """
     
-    def __init__(self,no_table=1, table='yappypar.tab'):
+    def __init__(self,no_table=1, table='yappypar.tab',**args):
         grammar= grules([
         ("G -> RULE G",self.GRule),
         ("G -> []",EmptySemRule),
@@ -1766,24 +1841,30 @@ class Yappy_grammar(Yappy):
         ("RH -> ID RH",self.RHRule),
         ("RH -> ID",self.RHRule),
         ("OPSEM -> []",self.OPSEMRule),
-        ("OPSEM -> semsym ID csemsym",self.OPSEMRule),#OPSEM->OP not allowed
-        ("OPSEM -> semsym ID OP csemsym",self.OPSEMRule),
+  #      ("OPSEM -> semsym ID csemsym",self.OPSEMRule),#OPSEM->OP not allowed
+  #      ("OPSEM -> semsym ID OP csemsym",self.OPSEMRule),
+      ("OPSEM -> IDS",self.OPSEMRule1),
+        ("OPSEM -> IDS OP",self.OPSEMRule1),
         ("OP -> opsym OPV",self.OPRule),
         ("OPV ->  ID  ID ", self.OPVRule) 
         ])
 
 
-        tokenize = [("\s+",""),
+        tokenize = [
+            ("\{\{.*\}\}",lambda x: ("IDS",string.strip(x[2:-2]))),
+                     ("\s+",""),
                     ("->",lambda x: ("rulesym",x)),
                     ("\|",lambda x: ("rulesep",x)),
                     (";",lambda x: ("ruleend",x)),
-                    ("}}",lambda x: ("csemsym",x)),
-                    ("{{",lambda x: ("semsym",x)),
+            #        ("}}",lambda x: ("csemsym",x)),
+            #        ("{{",lambda x: ("semsym",x)),
                     ("//",lambda x: ("opsym",x)),
                     (".*",lambda x: ("ID",x))]
-          
-        Yappy.__init__(self,tokenize,grammar,table,no_table)
-
+        if args.has_key('tmpdir'):
+                args1 = {'usrdir':string.rstrip(args['tmpdir'],'/')}
+        else:
+            args1 = {}        
+        Yappy.__init__(self,tokenize,grammar,table,no_table,**args1)
         
    
       
@@ -1794,7 +1875,7 @@ class Yappy_grammar(Yappy):
         except ValueError:
             raise SemanticError("Precedence must be an integer: %s given" %arg[0])
         if arg[1]!= 'left' and arg[1]!= 'right' and arg[1]!= 'noassoc':
-            raise SemanticError("Associativity must be 'left' or 'right' or 'noassoc': %s\
+           raise SemanticError("Associativity must be 'left' or 'right' or 'noassoc': %s\
         given" %arg[1])
         return (int(arg[0]),arg[1])
         
@@ -1809,6 +1890,15 @@ class Yappy_grammar(Yappy):
         if len(arg) == 0:
             return 'DefaultSemRule'
         
+    def OPSEMRule1(self,arg,context):
+        if len(arg) == 2:
+            return (arg[0],arg[1])
+        if len(arg) == 1:
+            return arg[0]
+        if len(arg) == 0:
+            return 'DefaultSemRule'
+        
+
     def RHRule(self,arg,context):
          if len(arg) == 1:
              return [arg[0]]
@@ -1831,7 +1921,8 @@ class Yappy_grammar(Yappy):
             if l == []: return (lhs,[],EmptySemRule)
             if type(l[1]) is  TupleType:
                 return (lhs,l[0],eval(l[1][0],globals(),context['locals']),l[1][1])
-            else: return (lhs,l[0],eval(l[1],globals(),context['locals']))
+            else:
+                return (lhs,l[0],eval(l[1],globals(),context['locals']))
         
         return map(lambda l:grule(self,l) ,arg[2])
         
@@ -1844,11 +1935,20 @@ class Yappy_grammar(Yappy):
 
     def test(self):
         st = """
-        reg -> reg + reg {{ DefaultSemRule// 200 left }} |
-        reg reg {{ DefaultSemRule // 200 left }} |
-        reg * {{ DefaultSemRule }} |
+        reg -> reg + reg {{DefaultSemRule}} // 200 left  |
+        reg reg {{DefaultSemRule}} // 200 left  |
+        reg * {{DefaultSemRule}} |
         ( reg ) {{DefaultSemRule}} |
-        id {{DefaultSemRule}} ;
+        id {{lambda l,c:l[0]}};
+        reg -> ;
+        a -> reg | reg ;
+        """
+        st1 = """
+        reg -> reg + reg {{DefaultSemRule // 200 left}}  |
+        reg reg {{DefaultSemRule // 200 left}}  |
+        reg * {{DefaultSemRule}} |
+        ( reg ) {{DefaultSemRule}} |
+        id {{DefaultSemRule}};
         reg -> ;
         a -> reg | reg ;
         """
